;;Macros and utilities
(ns rss-analyzer.utils
  (:gen-class))

(defn expand-home
  "Expands ~ to home directory"
  ;;modified from sauce: https://stackoverflow.com/questions/29585928/how-to-substitute-path-to-home-for
  [s]
  (if (clojure.string/starts-with? s "~")
    ;; (clojure.string/replace-first s "~" (System/getProperty "user.home"))
    (clojure.string/replace-first s "~" (System/getProperty "user.home"))
 s))

(defn expand-path
  [s]
  (if (clojure.string/starts-with? s "./")
    (clojure.string/replace-first s "." (System/getProperty "user.dir"))
    (expand-home s)))

(defn file-exists?
  "Returns true if file exists"
  [file]
  (let [path (if (vector? file)
               (first file)
               file)]
    (.exists (clojure.java.io/file (expand-path path)))))

(defn err-log
  "Writes message to stderr"
  [& args]
  (binding [*out* *err*]
    (apply println args)))

(defn fmap
  "Applies function [fun] to each ke-value pair from map"
  [fun mapa]
  ;;  Returns a new coll consisting of to-coll with all of the items of
  ;;  from-coll conjoined. A transducer may be supplied.
  (into
   (empty mapa)
   (for [[key val] mapa]
     [key (fun val)])))

(defmacro when-else
  "Evaluates condition and if it's not nil returns it's result
  When condition is nil, return [else] body

  Translates to (if pred
                     pred
                     else)
  [pred] is evaluated only once"
  [pred else]
  `(let [result# ~pred]
     (if result#
       result#
       ~else)))
